local font = CreateFont("farmerFont")
local playerName = UnitName("player")
local farmerFrame
local minDelay = 0.3
local specialList = {
  116158
}

local itemList = {}
local awaitingList = {}
local mapShown
local delay = 0
local _, chipName = GetItemInfoInstant(129100)
local events = {}
local slashCommands = {}

farmerFont:SetFont("Fonts\\FRIZQT__.TTF", 28, "thickoutline")
-- farmerFont:SetFont("Fonts\\skurri.TTF", 27, "thickoutline")
-- farmerFont:SetFont("Fonts\\ARIALN.TTF", 27, "thickoutline")

local function printTable (table)
  for i,v in pairs(table) do
    print(i, " - ", v)
  end
end

local function setTrueScale (frame, scale)
    frame:SetScale(1)
    frame:SetScale(scale / frame:GetEffectiveScale())
end

local function performAutoLoot ()
  -- for i = 1, GetNumLootItems(), 1 do
  for i = GetNumLootItems(), 1, -1 do
    LootSlot(i)
  end
end

local function handleItem (itemLink, count, texture)
  local itemName, itemLink_, itemRarity, itemLevel, itemMinLevel, itemType,
        itemSubType, itemStackCount, itemEquipLoc, iconFileDataID,
        itemSellPrice, itemClassID, itemSubVlassID, bindType, expacID,
        itemSetID, isCraftingReagent = GetItemInfo(itemLink)
  -- local itemId = GetItemInfoInstant(itemLink)
  local itemId = itemLink

  if (itemName == nil) then
    return
  end

  if (isCraftingReagent == true) then
    local icon = " |T" .. texture .. ":0|t "
    local totalCount = count + GetItemCount(itemId, true)

    farmerFrame:AddMessage(icon .. totalCount .. ' (' .. count .. ')' , 1, 1, 1, 1)
    return
  end

  if ((itemId >= 130200 and
       itemId <= 130204) or
      itemId == 129099) then
    local icon = " |T" .. texture .. ":0|t "
    local totalCount = count + GetItemCount(129100, true)

    farmerFrame:AddMessage(icon .. totalCount .. ' (' .. count .. ')' , 1, 1, 1, 1)
    return
  end

  for k, v in pairs(specialList) do
    if (itemId == v) then
      local icon = " |T" .. texture .. ":0|t "
      local totalCount = count + GetItemCount(itemId, true)

      farmerFrame:AddMessage(icon .. totalCount .. ' (' .. count .. ')' , 1, 0.85, 0, 1)
      return
    end
  end
end

--[[
///#############################################################################
/// Event listeners
///#############################################################################
]]--

function events:ADDON_LOADED (addon)
  if (addon == 'Farmer') then
    if (fastLoot == nil) then
      fastLoot = true
    end
  end
end

function events:LOOT_READY (lootSwitch)
  local loot

  if (GetTime() - delay < minDelay) then
    return
  end
  delay = GetTime()
  itemList = {}
  awaitingList = {}

  loot = GetLootInfo()

  for i, info in pairs(loot) do
    item = GetLootSlotLink(i)

    if (item ~= nil) then
      item = GetItemInfoInstant(item)
      itemList[i] = {}
      itemList[i]["item"] = item
      itemList[i]["quantity"] = info["quantity"]
      itemList[i]["texture"] = info["texture"]
    end
  end

  -- GetCVarBool("autoLootDefault")
  if (lootSwitch == 1) then
    mapShown = WorldMapFrame:IsShown()
    if (fastLoot == true) then
      performAutoLoot()
    end
  end
end

function events:LOOT_SLOT_CHANGED (slot)
  local texture, item, quantity, quality, locked = GetLootSlotInfo(slot)

  if (itemList[slot] ~= nil and
      quantity < itemList[slot]["quantity"]) then

    if (awaitingList[texture] == nil) then
      awaitingList[texture] = {}
      awaitingList[texture]["item"] = itemList[slot]["item"]
      awaitingList[texture]["quantity"] = itemList[slot]["quantity"] - quantity
    else
      awaitingList[texture]["quantity"] = awaitingList[texture]["quantity"] +
                                          itemList[slot]["quantity"] - quantity
    end
    -- print(itemList[slot]["quantity"])
    itemList[slot]["quantity"] = quantity
  end
end

function events:LOOT_SLOT_CLEARED (slot)
  if (itemList[slot] ~= nil) then
    if true then
      local texture, item, quantity, quality, locked = GetLootSlotInfo(slot)
    end

    local texture = itemList[slot]["texture"]

    if (awaitingList[texture] == nil) then
      awaitingList[texture] = {}
      awaitingList[texture]["item"] = itemList[slot]["item"]
      awaitingList[texture]["quantity"] = itemList[slot]["quantity"]
      -- print(itemList[slot]["quantity"])
    else
      awaitingList[texture]["quantity"] = awaitingList[texture]["quantity"] +
                                          itemList[slot]["quantity"]
    end
    itemList[slot] = nil
    -- itemList[slot]["quantity"] = 0
  end
end

function events:ITEM_PUSH (bag, texture)
  if (awaitingList[texture] ~= nil) then
    handleItem(awaitingList[texture]["item"],
               awaitingList[texture]["quantity"],
               texture)
    awaitingList[texture] = nil
  end
end

function events:LOOT_CLOSED ()
  if (mapShown == true and
      WorldMapFrame:IsShown() == false) then
    ToggleFrame(WorldMapFrame)
    mapShown = false
  end
end

local function eventHandler (self, event, ...)
  events[event](self, ...)
end

farmerFrame = CreateFrame("ScrollingMessageFrame", "farmerFrame", UIParent)
farmerFrame:SetWidth(250)
farmerFrame:SetHeight(200)
farmerFrame:SetSpacing(10)
farmerFrame:SetFrameStrata("MEDIUM")
farmerFrame:SetFrameLevel(2)
farmerFrame:SetTimeVisible(2)
farmerFrame:SetFadeDuration(0.5)
farmerFrame:SetMaxLines(7)
-- farmerFrame:SetInsertMode("BOTTOM")
farmerFrame:SetInsertMode("TOP")
farmerFrame:SetPoint("BOTTOM", UIParent, "CENTER", 0, 35)
farmerFrame:SetFontObject(font)
setTrueScale(farmerFrame, 1)
farmerFrame:Show()

farmerFrame:SetScript("OnEvent", eventHandler)

for k, v in pairs(events) do
  farmerFrame:RegisterEvent(k)
end

function slashCommands:fast ()
  fastLoot = true
  print('Enabled fast autoloot')
end

function slashCommands:nofast ()
  fastLoot = false
  print('Disabled fast autoloot')
end

local function slashHandler (command)
  command = string.lower(command)
  if (slashCommands[command] ~= nil) then
    slashCommands[command]()
    return
  end
  print('Farmer: unknown command "' .. command .. '"')
end

SLASH_FARMER1 = '/farmer'
SlashCmdList["FARMER"] = slashHandler
